# Jacob Library Setup for COM Interaction

This project requires the Jacob (Java COM Bridge) library to interact with Windows COM components. Follow these steps to set up the library:

## Steps to Set Up Jacob

1. **Create a lib directory** in the project root if it doesn't exist already:
   ```
   mkdir lib
   ```

2. **Download Jacob** from the official website or Maven repository:
   - Official website: https://github.com/freemansoft/jacob-project/releases
   
3. **Extract and copy** the following files to your project:
   - Copy `jacob.jar` to the `lib` folder
   - Copy the appropriate DLL for your system (32-bit or 64-bit) to a location on your system path:
     - `jacob-1.20-x86.dll` for 32-bit systems
     - `jacob-1.20-x64.dll` for 64-bit systems
     - `jacob-1.20-x64.dll` to C:\Windows\System32 for our case


4. **Set the Java library path** when running your application:
   ```
   java -Djava.library.path=path/to/dll -cp lib/jacob.jar;bin YourMainClass
   ```

## Alternative: Using Maven

If you're using Maven, add this dependency to your pom.xml:

```xml
<dependency>
    <groupId>net.sf.jacob-project</groupId>
    <artifactId>jacob</artifactId>
    <version>1.20</version>
</dependency>
```

## Important Notes

1. Jacob requires both the JAR file and the native DLL to work correctly.
2. The common.java file in this project uses Jacob to interact with SQLAcc.BizApp COM component.
3. Make sure the DLL matches your JVM architecture (32-bit or 64-bit).

## Additional System Requirements

- Windows operating system (Jacob only works on Windows)
- SQLAcc application installed on the system
- Proper permissions to access and control the COM components 