import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import com.jacob.com.Dispatch;
import com.jacob.com.Variant;

public class SQLAccountingGUI extends JFrame {
    private JTextField sqlQueryField;
    private JButton executeButton;
    private JButton loginButton;
    private JButton logoutButton;
    private JButton postDataButton;
    private JButton delDataButton;
    private JTable resultTable;
    private JScrollPane tableScrollPane;
    private JLabel statusLabel;

    public SQLAccountingGUI() {
        // Set up the frame
        setTitle("SQL Accounting Manager");
        setSize(800, 600);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLayout(new BorderLayout());

        // Create the components
        JPanel topPanel = new JPanel(new BorderLayout());

        // Login/Logout panel
        JPanel loginPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        loginButton = new JButton("Login");
        logoutButton = new JButton("Logout");
        logoutButton.setEnabled(false);
        loginPanel.add(loginButton);
        loginPanel.add(logoutButton);

        // Data manipulation panel
        JPanel dataPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        postDataButton = new JButton("Post Data (FAIRY)");
        delDataButton = new JButton("Delete Data (FAIRY)");
        postDataButton.setEnabled(false);
        delDataButton.setEnabled(false);
        dataPanel.add(postDataButton);
        dataPanel.add(delDataButton);

        // SQL query panel
        JPanel queryPanel = new JPanel(new BorderLayout());
        sqlQueryField = new JTextField();
        sqlQueryField.setText("SELECT * FROM AGENT WHERE ISACTIVE = 'True' ORDER BY CODE");
        executeButton = new JButton("Show Result");
        executeButton.setEnabled(false);
        queryPanel.add(new JLabel("SQL Query: "), BorderLayout.WEST);
        queryPanel.add(sqlQueryField, BorderLayout.CENTER);
        queryPanel.add(executeButton, BorderLayout.EAST);

        topPanel.add(loginPanel, BorderLayout.NORTH);
        topPanel.add(dataPanel, BorderLayout.CENTER);
        topPanel.add(queryPanel, BorderLayout.SOUTH);

        // Result table
        resultTable = new JTable();
        tableScrollPane = new JScrollPane(resultTable);

        // Status label
        statusLabel = new JLabel("Not connected");
        statusLabel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        // Add components to frame
        add(topPanel, BorderLayout.NORTH);
        add(tableScrollPane, BorderLayout.CENTER);
        add(statusLabel, BorderLayout.SOUTH);

        // Add action listeners
        loginButton.addActionListener(e -> loginToSystem());
        logoutButton.addActionListener(e -> logoutFromSystem());
        executeButton.addActionListener(e -> executeQuery());

        postDataButton.addActionListener(e -> {
            new SwingWorker<Void, Void>() {
                String action ="";
                @Override
                protected Void doInBackground() {
                    action = common.postData();
                    return null;
                }

                @Override
                protected void done() {
                    if (action.equals("Adding")) {
                        refreshTableData("added");
                    } else {
                        refreshTableData("edited");
                    }
                }
            }.execute();
        });

        delDataButton.addActionListener(e -> {
            new SwingWorker<Void, Void>() {
                @Override
                protected Void doInBackground() {
                    try {
                        Dispatch bizObject = Dispatch.call(common.comServer, "BizObjects").toDispatch();
                        Dispatch agent = Dispatch.call(bizObject, "Find", "Agent").toDispatch();
                        Variant docKey = Dispatch.call(agent, "FindKeyByRef", "CODE", "FAIRY");

                        if (docKey.isNull()) {
                            SwingUtilities.invokeLater(() -> JOptionPane.showMessageDialog(
                                    SQLAccountingGUI.this,
                                    "Record Not Found",
                                    "Delete Error",
                                    JOptionPane.INFORMATION_MESSAGE));
                        } else {
                            int confirm = JOptionPane.showConfirmDialog(
                                    SQLAccountingGUI.this,
                                    "Are you sure you want to delete the FAIRY record?",
                                    "Confirm Deletion",
                                    JOptionPane.YES_NO_OPTION);
                            if (confirm == JOptionPane.YES_OPTION) {
                                Dispatch.put(
                                        Dispatch.call(Dispatch.get(agent, "Params").toDispatch(), "Find", "Code")
                                                .toDispatch(),
                                        "Value", docKey);
                                Dispatch.call(agent, "Open");
                                Dispatch.call(agent, "Delete");
                                SwingUtilities.invokeLater(() -> refreshTableData("deleted"));
                            }
                        }
                    } catch (Exception ex) {
                        final String errorMsg = ex.getMessage();
                        SwingUtilities.invokeLater(() -> statusLabel.setText("Error during deletion: " + errorMsg));
                    }
                    return null;
                }
            }.execute();
        });

        // Clean up when window closes
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                logoutFromSystem();
            }
        });
    }

    private void loginToSystem() {
        statusLabel.setText("Attempting to login...");

        new SwingWorker<Boolean, Void>() {
            @Override
            protected Boolean doInBackground() {
                common.checkLogin();
                return common.comServer != null;
            }

            @Override
            protected void done() {
                try {
                    boolean success = get();
                    if (success) {
                        statusLabel.setText("Connected to SQLAccounting");
                        loginButton.setEnabled(false);
                        logoutButton.setEnabled(true);
                        executeButton.setEnabled(true);
                        postDataButton.setEnabled(true);
                        delDataButton.setEnabled(true);
                    } else {
                        statusLabel.setText("Login failed");
                    }
                } catch (Exception e) {
                    statusLabel.setText("Error: " + e.getMessage());
                }
            }
        }.execute();
    }

    private void logoutFromSystem() {
        try {
            if (common.comServer != null) {
                try {
                    Dispatch.call(common.comServer, "Logout");
                } catch (Exception e) {
                    statusLabel.setText("Error during logout: " + e.getMessage());
                }
                common.killApp();
                common.comServer = null;
            }

            statusLabel.setText("Logged out");
            loginButton.setEnabled(true);
            logoutButton.setEnabled(false);
            executeButton.setEnabled(false);
            postDataButton.setEnabled(false);
            delDataButton.setEnabled(false);

            // Clear table
            resultTable.setModel(new DefaultTableModel());
        } catch (Exception e) {
            statusLabel.setText("Error logging out: " + e.getMessage());
        }
    }

    private void executeQuery() {
        final String sql = sqlQueryField.getText().trim();
        if (sql.isEmpty()) {
            JOptionPane.showMessageDialog(this, "Please enter a SQL query", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        statusLabel.setText("Executing query...");
        executeButton.setEnabled(false);
        executeQueryAndUpdateTable(sql, "executed");
    }

    private void refreshTableData(String action) {
        final String sql = sqlQueryField.getText().trim();
        if (sql.isEmpty()) {
            return;
        }
        executeButton.setEnabled(false);
        statusLabel.setText("Refreshing data...");
        executeQueryAndUpdateTable(sql, action);
    }

    private void executeQueryAndUpdateTable(String sql, String action) {
        new SwingWorker<DefaultTableModel, Void>() {
            @Override
            protected DefaultTableModel doInBackground() {
                try {
                    Dispatch dbManager = Dispatch.get(common.comServer, "DBManager").toDispatch();
                    Dispatch dataset = Dispatch.call(dbManager, "NewDataSet", sql).toDispatch();

                    // Convert dataset to table model
                    DefaultTableModel model = new DefaultTableModel();

                    Variant recordCount = Dispatch.get(dataset, "RecordCount");
                    if (recordCount.getInt() > 0) {

                        Dispatch fields = Dispatch.get(dataset, "Fields").toDispatch();
                        int fieldCount = Dispatch.get(fields, "Count").getInt();

                        String[] columnNames = new String[fieldCount];
                        for (int i = 0; i < fieldCount; i++) {
                            Dispatch field = Dispatch.call(fields, "Items", i).toDispatch();
                            columnNames[i] = Dispatch.get(field, "FieldName").getString();
                        }
                        model.setColumnIdentifiers(columnNames);


                        while (!Dispatch.get(dataset, "EOF").getBoolean()) {
                            Object[] rowData = new Object[fieldCount];
                            for (int i = 0; i < fieldCount; i++) {
                                Dispatch field = Dispatch.call(fields, "Items", i).toDispatch();
                                String fieldName = Dispatch.get(field, "FieldName").getString();
                                Dispatch foundField = Dispatch.call(dataset, "FindField", fieldName).toDispatch();
                                rowData[i] = Dispatch.get(foundField, "AsString").getString();
                            }
                            model.addRow(rowData);
                            Dispatch.call(dataset, "Next");
                        }
                    }
                    return model;
                } catch (Exception e) {
                    final String errorMsg = e.getMessage();
                    SwingUtilities.invokeLater(() -> statusLabel.setText("Error executing query: " + errorMsg));
                    return new DefaultTableModel();
                }
            }

            @Override
            protected void done() {
                try {
                    DefaultTableModel model = get();
                    resultTable.setModel(model);
                    statusLabel.setText("Data " + action + " successfully. Records: " + model.getRowCount());
                } catch (Exception e) {
                    statusLabel.setText("Error processing results: " + e.getMessage());
                } finally {
                    executeButton.setEnabled(true);
                }
            }
        }.execute();
    }

    public static void main(String[] args) {

        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
        }

        // Create and show the GUI
        SwingUtilities.invokeLater(() -> {
            SQLAccountingGUI gui = new SQLAccountingGUI();
            gui.setVisible(true);
        });
    }
}